package io.resourcepool.ssdp.model;

/**
 * This class represents the SsdpClient Options.
 * Defaults are shown in static final fields.
 */
public class SsdpClientOptions {

  private Boolean ignoreInterfaceDiscoveryErrors = false;
  private Boolean lookupAllIncomingAnnouncements = true;
  /**
   * Allow the program to process the discover response
   */
  private Boolean useCache=true;

  public Boolean getIgnoreInterfaceDiscoveryErrors() {
    return ignoreInterfaceDiscoveryErrors;
  }

  public Boolean getLookupAllIncomingAnnouncements() {
    return lookupAllIncomingAnnouncements;
  }

  public Boolean getUseCache() {
    return useCache;
  }
// BEGIN GENERATED CODE

  public static Builder builder() {
    return new Builder();
  }

  public static final class Builder {

    private Boolean ignoreInterfaceErrors = false;
    private Boolean lookupAllIncomingAnnouncements = true;
    private Boolean useCache=true;

    private Builder() {
    }

    /**
     * Ignore Interface Errors.
     * Specifically aimed at fails happening when joining group on all interfaces.
     * Default behaviour is to fail on first error.
     * Using this method will silently ignore those errors and proceed with other interfaces.
     * All reported errors can be retrieved by overriding the onFailedAndIgnored method in the DiscoveryListener.
     * @see io.resourcepool.ssdp.model.DiscoveryListener#onFailedAndIgnored(Exception)
     * @return the current builder
     */
    public Builder ignoreInterfaceDiscoveryErrors() {
      this.ignoreInterfaceErrors = true;
      return this;
    }

    /**
     * Disable Auto Lookup.
     * When receiving incoming announcements, the default behaviour is to send a discovery request to look it up.
     * This behaviour can be disabled with this method.
     * @return the current builder
     */
    public Builder disableAutoLookup() {
      this.lookupAllIncomingAnnouncements = false;
      return this;
    }
    public Builder disableCache() {
      this.useCache = false;
      return this;
    }
    public io.resourcepool.ssdp.model.SsdpClientOptions build() {
      io.resourcepool.ssdp.model.SsdpClientOptions discoveryOptions = new io.resourcepool.ssdp.model.SsdpClientOptions();
      discoveryOptions.ignoreInterfaceDiscoveryErrors = this.ignoreInterfaceErrors;
      discoveryOptions.useCache=this.useCache;
      discoveryOptions.lookupAllIncomingAnnouncements=this.lookupAllIncomingAnnouncements;
      return discoveryOptions;
    }
  }

  // END GENERATED CODE
}
